;;; -*- Mode: Lisp; Package: RDOC -*-

;;;  (c) copyright 2002 by
;;;           Joachim P., Jrme P., Hugh P., Matthias R.
;;;           (restordoc@emi.u-bordeaux.fr)
;;;
;;; This library is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU Library General Public
;;; License as published by the Free Software Foundation; either
;;; version 2 of the License, or (at your option) any later version.
;;;
;;; This library is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; Library General Public License for more details.
;;;
;;; You should have received a copy of the GNU Library General Public
;;; License along with this library; if not, write to the Free
;;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
;;; 02111-1307 USA.

(in-package :RDOC)

(defclass canvas-pane (basic-gadget)
  ((image :type image 
	  :initform nil
	  :accessor canvas-image)
   (zoom :initform 1) 
   (center-zoom-x :initform 0)
   (center-zoom-y :initform 0)
   (translate-x :initform 0)
   (translate-y :initform 0) 
   (button :initform nil
	   :reader canvas-button)
   (old-x :initform 0)
   (old-y :initform 0)
   (canvas-pixmap :initform nil)
   (selected-region :initform nil
		    :reader canvas-selected-region)))

(defmethod compose-space ((pane canvas-pane))
  (make-space-requirement :width 300 :height 300))

;;; Transmit canvas's keys events to minibuffer if it's wait for input
;;; else catches keystrokes from event loop
(defmethod handle-event ((pane canvas-pane) (event key-press-event))
  (if (rdoc-mini-buffer-active-p *application-frame*)
      (handle-event (rdoc-mini-buffer *application-frame*) event)
      (my-keypress-handler (keyboard-event-key-name event)
			   (event-modifier-state event))))

(defmacro with-canvas-image (image &body body)
  `(let* ((pic (when image (image-pixels ,image)))      
	  (height (if pic (car (array-dimensions pic)) 0))
	  (width (if pic (cadr (array-dimensions pic)) 0)))
    ,@body))

;;; Make sure min <= x <= max
;(defmacro crop-it (x min max)
;  `(cond ((< ,x ,min) (setf ,x ,min))
;	 ((> ,x ,max) (setf ,x ,max))))
(defmacro crop-it (x min max)
  `(setf ,x (round (max ,min (min ,x ,max)))))


;;; Calculate and apply transformation to sheet's medium
(defmethod set-transformation ((pane canvas-pane))
  (with-slots (zoom center-zoom-x center-zoom-y 
	       translate-x translate-y image) pane     
    (setf (medium-transformation (sheet-medium pane))
	  (compose-transformations 
	   (make-translation-transformation translate-x 
					    translate-y)
	   (make-scaling-transformation* zoom zoom 
					 center-zoom-x 
					 center-zoom-y)))))

(defmacro with-tool (pane &body body)
  `(with-slots (zoom translate-x translate-y old-x old-y image button) ,pane
    (let ((transform (medium-transformation (sheet-medium pane))))
      (with-canvas-image image
	(multiple-value-bind (tx ty) 
	    (untransform-position transform pos-x pos-y)
	  (if (or (< tx 0) (>= tx width)
		  (< ty 0) (>= ty height))
	      (setf tx -1 ty -1)
	      (setf tx (floor tx)
		    ty (floor ty)))
	  ,@body)))))

(defun reset-canvas (canvas-pane)
  (with-slots (zoom center-zoom-x center-zoom-y 
	       translate-x translate-y) canvas-pane
    (setf zoom 1
	  center-zoom-x 0
	  center-zoom-y 0
	  translate-x 0
	  translate-y 0))
  (set-transformation canvas-pane)
  (dispatch-repaint canvas-pane nil))

(defmethod (setf canvas-image) :after (image (pane canvas-pane))
  (declare (ignore image))
  (reset-canvas pane))

;; Events

(defmethod dispatch-repaint ((pane canvas-pane) region)
  (handle-repaint pane region))

(defmethod handle-repaint ((sheet canvas-pane) region)
  (unless region
    (setf region (sheet-region sheet)))
  (unless (eq region +nowhere+)
    (with-slots (image) sheet
      (multiple-value-bind (x1 y1 x2 y2) (bounding-rectangle* region)
	(draw-rectangle* sheet x1 y1 x2 y2 
			 :ink (make-gray-color .45) 
			 :filled t
			 :clipping-region region
			 :transformation 
			 (invert-transformation 
			  (medium-transformation (sheet-medium sheet)))))
      (when image
	(draw-image sheet image :clipping-region region)))
    (mapc (lambda (tool) ; call repaint tool callbacks
	    (apply-tool-repaint sheet tool region))
	  *rdoc-tool-list*)))

(defmethod handle-event :around ((pane t) (event t))
  (let ((*application-frame* (pane-frame pane)))
    (call-next-method pane event)))

(defmethod dispatch-event ((pane canvas-pane) event)
  (handle-event pane event))

(defmacro call-apply-tool (pane tool-event event)
  `(when (canvas-image ,pane)
     (apply-tool ,pane 
                 (rdoc-current-tool *application-frame*)
                 ,tool-event
                 (canvas-button ,pane)
                 (pointer-event-x ,event) 
                 (pointer-event-y ,event))))

(defmethod handle-event ((pane canvas-pane)
			 (event pointer-button-press-event))
  (setf (slot-value pane 'button) (pointer-event-button event))
  (call-apply-tool pane :pointer-button-press event))

(defmethod handle-event ((pane canvas-pane) 
			 (event pointer-motion-event))
  (call-apply-tool pane :pointer-motion event))

(defmacro update-old-pos ()
  `(with-slots (old-x old-y) pane
    (setf old-x (pointer-event-x event)
          old-y (pointer-event-y event))))

(defmethod handle-event :after ((pane canvas-pane) 
				(event pointer-motion-event))
  (update-old-pos))

(defmethod handle-event :after ((pane canvas-pane) 
				(event pointer-button-press-event))
  (update-old-pos))

(defmethod handle-event ((pane canvas-pane) 
			 (event pointer-button-release-event))
  (call-apply-tool pane :pointer-button-release event)
  (setf (slot-value pane 'button) nil))

;; Default tool callbacks

(defgeneric apply-tool (pane tool event button pos-x pos-y))

(defgeneric apply-tool-repaint (pane tool region))

(defmethod apply-tool-repaint ((pane canvas-pane) tool region)
  (declare (ignore tool region)))

(defmethod apply-tool :before ((pane canvas-pane) tool
			       (event (eql :pointer-button-press)) 
			       button pos-x pos-y)
  (declare (ignore button pos-x pos-y))
  (setf-status-line :infos (format nil "~A" tool)))

(defmethod apply-tool :before ((pane canvas-pane) tool
			       (event (eql :pointer-motion))
			       button pos-x pos-y)
  (declare (ignore tool button))
  (with-tool pane
    (setf-status-line :zoom zoom
		      :pos-x tx
		      :pos-y ty
		      :color (when (>= tx 0) 
			       (aref pic ty tx)))))

(defmethod apply-tool :before ((pane canvas-pane) tool
			       (event (eql :pointer-button-release)) 
			       button pos-x pos-y)
  (declare (ignore tool button pos-x pos-y))
  (setf-status-line :infos ""))

(defmethod apply-tool ((pane canvas-pane) tool
		       event button pos-x pos-y)
  (declare (ignore tool event button pos-x pos-y)))

(defmacro define-tool ((tool event button) &body body)
  (if (eql tool t)
      `(defmethod apply-tool ((pane canvas-pane) 
			      tool
			      (event (eql ,event))
			      (button (eql ,button)) 
			      pos-x pos-y)
	,@body)
      `(defmethod apply-tool ((pane canvas-pane) 
			      (tool (eql ,tool))
			      (event (eql ,event))
			      (button (eql ,button)) 
			      pos-x pos-y)
	,@body)))

;;Pixmap functions

(defmacro with-pixmap-dimensions ((pane) &body body)
  (let ((old-trans (gensym)))
    `(let ((,old-trans (medium-transformation (sheet-medium ,pane))))
       (setf (medium-transformation (sheet-medium pane)) 
	     +identity-transformation+)
       (let ((pixmap-width 
	      (round (bounding-rectangle-width (sheet-region ,pane))))
	     (pixmap-height 
	      (round (bounding-rectangle-height (sheet-region ,pane)))))
	 ,@body)
       (setf (medium-transformation (sheet-medium ,pane)) ,old-trans))))
	 
(defmacro get-canvas-pixmap ()
  `(with-slots (canvas-pixmap) pane
    (with-pixmap-dimensions (pane)
      (setf canvas-pixmap (allocate-pixmap pane pixmap-width pixmap-height))
      (copy-to-pixmap pane 
		      0 0 pixmap-width pixmap-height canvas-pixmap))))

(defmacro put-canvas-pixmap ()
  `(with-slots (canvas-pixmap) pane
    (with-pixmap-dimensions (pane)
      (copy-from-pixmap canvas-pixmap 
			0 0 pixmap-width pixmap-height pane 0 0))))





