;;; -*- Mode: Lisp; Package: RDOC -*-

;;;  (c) copyright 2002 by
;;;           Joachim P., Jrme P., Hugh P., Matthias R.
;;;           (restordoc@emi.u-bordeaux.fr)
;;;  (c) copyright 2001 by
;;;           Aymeric Lacorte (lacorte@emi.u-bordeaux.fr)
;;;           Iban Hatchondo (hatchond@emi.u-bordeaux.fr)
;;;           Frederique Ferrere (ferrere@emi.u-bordeaux.fr)
;;;           Julien Boninfante (boninfan@emi.u-bordeaux.fr)
;;;           Robert Strandh (strandh@labri.u-bordeaux.fr)
;;;
;;; This library is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU Library General Public
;;; License as published by the Free Software Foundation; either
;;; version 2 of the License, or (at your option) any later version.
;;;
;;; This library is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; Library General Public License for more details.
;;;
;;; You should have received a copy of the GNU Library General Public
;;; License along with this library; if not, write to the Free
;;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA

(in-package :RDOC)

;;; char with no modifier
(defun N- (char)
  (char-int char))

;;; char with control modifier
(defun C- (char)
  (let ((c (char-int char)))
    (when (and (>= c 96) (<= c 127))
      (setf c (- c 32)))
    (+ 256 c)))

;;; char with meta modifier
(defun M- (char)
  (let ((c (char-int char)))
    (+ 512 c)))

;;; char with meta and control modifiers
(defun MC- (char)
  (let ((c (char-int char)))
    (when (and (>= c 96) (<= c 127))
      (setf c (- c 32)))
    (+ 512 256 c)))

(defun superchar-reader (stream char)
  (declare (ignore char))
  (let ((first (read-char stream nil #\N t))
	(second (read-char stream nil #\N t))
	(third (read-char stream nil #\N t)))
    (ecase first
      (#\N (N- third))
      (#\C (C- third))
      (#\M (if (char= second #\C)
	       (MC- (read-char stream nil #\N t))
	       (M- third))))))

(set-macro-character #\% #'superchar-reader)

;;; An entry in a keymap can be another keymap or 
;;; a symbol naming a function
(defvar *global-keymap* (make-hash-table))

;;; Associate a key with a (function-) symbol in a keymap.
;;; If a prefix entry in the keymap is not a keymap, replace
;;; that entry with a keymap.
(defun set-key (keymap key symbol)
;;;  (gassert (listp key) "key must be a list: " key)
  (do ()
      ((null (cdr key)))
    (unless (hash-table-p (gethash (car key) keymap))
      (setf (gethash (car key) keymap) (make-hash-table)))
    (setf keymap (gethash (car key) keymap))
    (setf key (cdr key)))
  (setf (gethash (car key) keymap) symbol))

(defmacro set-global-key (key symbol)
  `(set-key *global-keymap* ,key ,symbol))

;;; Take a keymap and a key (a list of super characters) and 
;;; return the value of the key in the keymap.  For success, 
;;; either another keymap or a symbol is returned.  For failure, 
;;; nil is returned. 
(defun lookup-key (keymap key)
  (cond ((null key) keymap)
	((not (hash-table-p keymap)) nil)
	(t (lookup-key (gethash (car key) keymap) (cdr key)))))

;;; Compute a super character from a state and two key codes.
;;; The super character will have a value between 0 and 65535
;;; and is computed as follows:
;;;   - If the modifier contains shift or capslock, then the second 
;;;     code is the base code, otherwise the first code is the base
;;;     code.
;;;   - The most significant byte of the superchar is the modifier and 
;;;     reflects the modifier keys. This byte encodes a bit vector where 
;;;     each bit corresponds to an element in the vector.
;;;     For now we only support control (1) and meta (2).
;;;   - If the modifier contains control, then the LSB does not have 
;;;     a value between 96 and 127.  Should the base char contain any
;;;     of these values, we subtract 32 from it. 
(defun compute-superchar (state char1 char2)
  (let ((base (if (or (logbitp 0 state) (logbitp 1 state)) char2 char1))
	(mod 0))
    (when (and (>= base 96) (<= base 127) (logbitp 2 state))
      (setf base (- base 32)))
    (when (logbitp 2 state)
      (setf mod (logior mod 1)))
    (when (logbitp 6 state)
      (setf mod (logior mod 2)))
    (+ (* mod 256) base)))

;;; This one should be altered whenever we receive a keymap
;;; change event
(defvar *X-keymap*)

(defun my-keypress-handler (character state)
 (let* ((keysyms (car (xlib::character->keysyms character *display*)))
	 (code (xlib::keysym->keycodes *display* keysyms))
	 (c1 (aref *X-keymap* code 0))
	 (c2 (aref *X-keymap* code 1)))
    (cond ((or (>= c1 256) (>= c2 256)) nil)
	  (t (compute-superchar state c1 c2)))))

(defun lookup-key-in-active-keymaps (key)
  (let ((global-content (lookup-key *global-keymap* key)))
    (cond ((and global-content (symbolp global-content)) global-content)
	  (t nil))))

(defun key-undefined (key)
  (let ((global-content (lookup-key *global-keymap* key)))
    (not global-content)))

(defvar *event-buffer* nil)

(defun unread-event (rdoc-frame-event)
  (setf *event-buffer* rdoc-frame-event))

(defun read-event (frame)
  (let ((sheet-buffer (slot-value frame 'clim::pane)))
    (cond (*event-buffer* (prog1 *event-buffer* 
			    (setf *event-buffer* nil)))
	  (t (or (rdoc-event-read sheet-buffer) 
		 (throw 'cmd-loop nil))))))

(defun read-key-sequence (frame)
  (let ((key-so-far (list (read-event frame))))
    (lookup-key-in-active-keymaps key-so-far)
    (when (key-undefined key-so-far)
      (throw 'cmd-loop nil))
    key-so-far))

(defun read-key-and-call-command (frame)
  (let ((commande (lookup-key-in-active-keymaps (read-key-sequence frame))))
    (funcall commande)))

(defun rdoc-event-read (sheet)
  (loop with event = nil
	with result = nil
	while (not (and event 
			(not (when (eq event :timeout) 
			       (throw 'timeout :timeout)))
			(car (push (distribute-event (port sheet) event)
				   result))))
        do (setf event (climi::get-next-event (port sheet) :timeout 0))
	finally (return (car result))))
