;;; -*- Mode: Lisp; Package: RDOC -*-

;;;  (c) copyright 2002 by
;;;           Joachim P., Jrme P., Hugh P., Matthias R.
;;;           (restordoc@emi.u-bordeaux.fr)
;;;
;;; This library is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU Library General Public
;;; License as published by the Free Software Foundation; either
;;; version 2 of the License, or (at your option) any later version.
;;;
;;; This library is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; Library General Public License for more details.
;;;
;;; You should have received a copy of the GNU Library General Public
;;; License along with this library; if not, write to the Free
;;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA

(in-package :RDOC)

(defvar *ask-module-parameters* nil)

;;; Catch the SHIFT key in the menu-bar
(defmethod handle-event ((pane climi::menu-bar) (event device-event))
  (setf *ask-module-parameters* (eq (logand (climi::event-modifier-state event) 
					    1)
				    1)))

(defstruct module
  ;; Menu group name
  group
  ;; Submenu entry name
  name
  command
  ;; Parameters is a string of parameters for command after the pix-array
  parameters
  ;; Keystroke defined as in keystrokes.lisp
  keystroke
  callback)

;;; A global hash-table which contains modules
;;; The key is a string generated by module-full-name
;;; from module's group & name 
(defvar *rdoc-modules* nil)

(defmacro module-full-name (group name)
  `(concatenate 'string ,group "/" ,name))

;;; Generate a group-command-table sympbol
(defmacro generate-group-command-table (group)
  `(intern (concatenate 'string ,group "-module-command-table")))

(defmacro make-module-process (canvas module pixels)
  `(let* ((selected-region (canvas-selected-region ,canvas))
	   (full-name (module-full-name (module-group ,module) 
					(module-name ,module)))
	   (frame (make-application-frame 'rdoc-frame 
					  :pretty-name (format nil
							       "RestorDoc - ~A" 
							       full-name)))
	   (parameters (module-parameters ,module))
	   (command (module-command ,module))
	   (pixels (copy-array ,pixels))
	   (selected-pixels nil))
     (rdoc-frame-init frame)
     (when selected-region
       (multiple-value-bind (x1 y1 x2 y2) 
	   (rectangle-edges* selected-region)
	 (setf selected-pixels 
	       (copy-sub-array-from-array pixels 
					  (- x2 x1) (- y2 y1)
					  x1 y1))))   
     (rdoc-frame-init frame)
     (setf (canvas-image (rdoc-canvas frame)) 
	   (make-256-gray-level-image pixels))
     (let ((start-time (get-internal-real-time)))
       (apply command (cons (if selected-region 
				selected-pixels 
			        pixels)
			    (read-from-string (format nil "(~A)" 
						      parameters))))
       (setf start-time (/ (- (get-internal-real-time)
			      start-time)
			   1000.0))
       (write-string (format nil "~A seconds" start-time) 
		     (rdoc-mini-buffer frame)))
     (when selected-region
       (copy-sub-array-to-array selected-pixels pixels 
				(rectangle-min-x selected-region)
				(rectangle-min-y selected-region)))
     (run-frame-top-level frame)))

(defmacro make-module-callback (module)
  `(let* ((canvas (rdoc-canvas *application-frame*))
	  (image (canvas-image canvas))
	  (pixels (when image (clim-clx::image-pixels image))))
     (when pixels
       (when *ask-module-parameters*
	 (setf *ask-module-parameters* nil)
	 (write-mini-buffer "Enter module parameters :")
	 (let ((new-params (read-mini-buffer :default-string 
					     (module-parameters ,module))))
	   (unless (string= "" new-params)
	     (setf (module-parameters ,module) new-params))))
       (climi::make-process (lambda () 
			      (make-module-process canvas ,module pixels))))))

(defun insert-module (group name command &key (parameters "") (keystroke nil))
  "Insert a module in RestorDoc"
  (unless *rdoc-modules*
    (setf *rdoc-modules* (make-hash-table :test #'equal)))
  (let ((module (make-module :group group 
			     :name name 
			     :command command 
			     :parameters parameters
			     :keystroke keystroke))
	(group-table (generate-group-command-table group))
	(full-name (module-full-name group name)))
    (setf (gethash full-name *rdoc-modules*) module)
    (unless (find-command-table group-table :errorp nil)
      (make-command-table group-table :errorp nil)
      (add-menu-item-to-command-table 'modules-command-table group 
				      :menu group-table 
				      :errorp nil))
    (let ((callback (lambda () 
		      (make-module-callback module))))
      (setf (module-callback module) callback)
      (add-menu-item-to-command-table group-table name
				      :command callback
				      :keystroke keystroke
				      :errorp nil)
      (set-global-key keystroke callback))
    (values group name)))

(defun remove-module (group name)
  "Remove a module from RestorDoc"
  (let* ((full-name (module-full-name group name))
	 (group-table (generate-group-command-table group))
	 (module (gethash full-name *rdoc-modules*)))
    (when module
      (remove-menu-item-from-command-table group-table name)
      (set-global-key (module-keystroke module) nil)
      (remhash name *rdoc-modules*)
      t)))

(defun update-module (group name &key command parameters)
  "Change the command or the parameters associed to a module"
  (let ((module (gethash (module-full-name group name) *rdoc-modules*)))
    (when module
      (when command
	(setf (module-command module) command))
      (when parameters
	(setf (module-parameters module) parameters))
      t)))

(defun apply-module (group name &key parameters)
  "Apply the module function to current canvas's image"
  (let ((module (gethash (module-full-name group name) *rdoc-modules*)))
    (when module
      (update-module group name :parameters parameters)
      (funcall (module-callback module)))))


