;;; -*- Mode: Lisp; Package: RDOC -*-

;;;  (c) copyright 2002 by
;;;           Joachim P., Jrme P., Hugh P., Matthias R.
;;;           (restordoc@emi.u-bordeaux.fr)
;;;
;;; This library is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU Library General Public
;;; License as published by the Free Software Foundation; either
;;; version 2 of the License, or (at your option) any later version.
;;;
;;; This library is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; Library General Public License for more details.
;;;
;;; You should have received a copy of the GNU Library General Public
;;; License along with this library; if not, write to the Free
;;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
;;; 02111-1307 USA.

(in-package :RDOC)

;; A pane to show a graylevel color - palette

(defclass gray-pane (basic-gadget)
  ((gray-level :initform 0
	       :initarg :gray-level
	       :accessor gray-level)))

(defmethod compose-space ((gadget gray-pane))
  (make-space-requirement :min-width 50
			  :width 50
			  :min-height 50
			  :height 50))

(defmethod handle-event ((pane gray-pane) (event window-repaint-event))
  (handle-repaint pane (sheet-region pane)))

(defmacro make-256-gray-color (gray-level)
  `(make-gray-color (/ ,gray-level 255)))

(defmethod handle-repaint ((pane gray-pane) region)
  (declare (ignore region))
  (multiple-value-bind (x1 y1 x2 y2) 
      (bounding-rectangle* (sheet-region pane))
    (draw-rectangle* pane x1 y1 x2 y2 
		     :ink (make-256-gray-color (gray-level pane))
		     :filled t)))

(defmethod (setf gray-level) :after (gray-level (pane gray-pane))
  (declare (ignore gray-level))
  (handle-repaint pane (sheet-region pane)))

(defun get-palette-color ()
  (gray-level (rdoc-color-button *application-frame*)))

(defun get-palette-color-object ()
  (make-256-gray-color (get-palette-color)))

(defun set-palette-color (gray-level)
  (setf (gadget-value (rdoc-color-slider *application-frame*)) 
	gray-level
	(gray-level (rdoc-color-button *application-frame*)) 
	(round gray-level)))

(defun color-slider-callback (gadget value)
  (declare (ignore gadget))
  (set-palette-color value))

;; Status line

(defclass status-line-pane (text-field-pane)
  ((status-filename :initform "noname"
		    :reader status-filename) 
   (status-infos :initform ""
		 :reader status-infos)
   (status-zoom :initform 1)
   (status-pos-x :initform 0)
   (status-pos-y :initform 0)
   (status-color :initform 0)))

(defmethod format-status-line ((pane status-line-pane))
  (with-slots (status-filename status-infos 
	       status-zoom status-pos-x status-pos-y 
	       status-color) pane
    (format nil "~A ~A- (Zoom ~D) - (~A-~D) - (Color ~D)" 
	    status-filename status-infos 
	    status-zoom status-pos-x status-pos-y 
	    status-color)))

(defmethod update-status-line ((pane status-line-pane))
  (setf (gadget-value pane) (format-status-line pane)))

(defmacro when-setf (x v)
  `(when ,v
     (setf ,x ,v)))

(defun setf-status-line (&key 
			 filename
			 infos
			 zoom 
			 pos-x
			 pos-y
			 color)
  (let ((status-pane (rdoc-status *application-frame*)))
    (with-slots (status-filename status-infos
		 status-zoom status-pos-x status-pos-y status-color) 
	status-pane
      (when-setf status-filename filename)
      (when-setf status-infos infos)
      (when-setf status-zoom zoom)
      (when-setf status-pos-x pos-x)
      (when-setf status-pos-y pos-y)
      (when-setf status-color color))
    (update-status-line status-pane)))

;;; Tool button (toggle)

;;; Define an exclusive class of toggle 
;;; buttons designed like push-buttons

;; A list of all tools
(defvar *rdoc-tool-list* nil)

(defclass rdoc-tool-button-pane  (toggle-button-pane) 
  ()
  (:default-initargs
    :align-x :center
    :align-y :center
    :x-spacing 4
    :y-spacing 4))

(defmethod handle-repaint ((pane rdoc-tool-button-pane) region)
  (declare (ignore region))
  (when (sheet-grafted-p pane)
    (climi::with-special-choices (pane)
      (with-slots (climi::armed) pane
	(multiple-value-bind (x1 y1 x2 y2) 
	    (bounding-rectangle* (sheet-region pane))
	  (draw-rectangle* pane x1 y1 x2 y2 
			   :ink (climi::effective-gadget-background pane))
	  (climi::draw-bordered-rectangle* pane x1 y1 x2 y2
					   :style 
					   (if (or (climi::gadget-value pane) 
						   climi::armed) 
					       :inset 
					     :outset))
	  (let ((dx (+ climi::*3d-border-thickness* 
		       (climi::pane-x-spacing pane)))
		(dy (+ climi::*3d-border-thickness* 
		       (climi::pane-y-spacing pane))))
	    (climi::draw-label* pane (+ x1 dx) (+ y1 dy) (- x2 dx) (- y2 dy)
				:ink
				(climi::effective-gadget-foreground pane))))))))

(defmethod compose-space ((gadget rdoc-tool-button-pane))
  (space-requirement+* 
   (space-requirement+* (climi::compose-label-space gadget)
			:min-width (* 2 (climi::pane-x-spacing gadget))
			:width (* 2 (climi::pane-x-spacing gadget))
			:max-width (* 2 (climi::pane-x-spacing gadget))
			:min-height (* 2 (climi::pane-y-spacing gadget))
			:height (* 2 (climi::pane-y-spacing gadget))
			:max-height (* 2 (climi::pane-y-spacing gadget)))
   :min-width (* 2 climi::*3d-border-thickness*)
   :width (* 2  climi::*3d-border-thickness*)
   :max-width (* 2 climi::*3d-border-thickness*)
   :min-height (* 2 climi::*3d-border-thickness*)
   :height (* 2 climi::*3d-border-thickness*)
   :max-height (* 2 climi::*3d-border-thickness*)))

(defun make-tool-button (label tool &key callback value (width 100) (height 50))
  (push tool *rdoc-tool-list*)
  (setf (gethash tool (rdoc-tools *application-frame*)) 
	(make-pane 'rdoc-tool-button-pane
		   :label label
		   :indicator-type :one-of
		   :value value
		   :width width :height height 
		   :value-changed-callback (lambda (gadget value)
					     (declare (ignore gadget value))
					     (set-current-tool tool)
					     (when callback
					       (funcall callback)))
		   :name tool)))

;;; Unselect the current tool before selecting the new one
(defun set-current-tool (tool)
  (let* ((tools (rdoc-tools *application-frame*))
	 (old-tool (gethash (rdoc-current-tool *application-frame*) tools)) 
	 (current-tool (gethash tool tools)))
    (setf (climi::gadget-value old-tool) nil
	  (rdoc-current-tool *application-frame*) tool
	  (climi::gadget-value current-tool) t)))
