;;; -*- Mode: Lisp; Package: RDOC -*-

;;;  (c) copyright 2002 by
;;;           Joachim P., Jrme P., Hugh P., Matthias R.
;;;           (restordoc@emi.u-bordeaux.fr)
;;;
;;; This library is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU Library General Public
;;; License as published by the Free Software Foundation; either
;;; version 2 of the License, or (at your option) any later version.
;;;
;;; This library is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; Library General Public License for more details.
;;;
;;; You should have received a copy of the GNU Library General Public
;;; License along with this library; if not, write to the Free
;;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
;;; 02111-1307 USA.

(in-package :RDOC)

;;; Initialization file for module loading
(defvar *rdoc-init-file* ".restordoc.lisp")

(defvar *display*)

(defun rdoc ()
  (loop for port in climi::*all-ports*
      do (destroy-port port))
  (setq climi::*all-ports* nil)
  (let ((appframe (make-application-frame 'rdoc-frame 
					  :pretty-name "RestorDoc")))
    (rdoc-frame-init appframe)
    (rdoc-init)
    (run-frame-top-level appframe)))

;;; Search for a typed sheet inside a named parent sheet   
(defun find-pane-of-type-in-multiple-sheet (frame pane-name pane-type)
  (climi::find-pane-of-type (sheet-children 
			     (climi::find-pane-named frame pane-name)) 
			    pane-type))

;;; Initialize frame's slots
(defun rdoc-frame-init (frame)
  (with-slots (status canvas interactor mini-buffer 
	       color-slider color-button) frame
  (setf *display* (clim-clx::clx-port-display (clim-internals::port frame))
	*X-keymap* (xlib:keyboard-mapping *display*)
	status (climi::find-pane-named frame 
				       'status)
	interactor (find-pane-of-type-in-multiple-sheet frame 
							'interactor 
							'interactor-pane)
	mini-buffer (find-pane-of-type-in-multiple-sheet frame 
							 'mini-buffer 
							 'interactor-pane)
	canvas (climi::find-pane-named frame 
				       'canvas)
	color-slider (climi::find-pane-named frame 
					     'color-slider)
	color-button (climi::find-pane-named frame 
					     'color-button))))

(defun rdoc-init ()
  (when (probe-file *rdoc-init-file*)
    (load *rdoc-init-file*)))

(defun load-image (filename)
  (let ((image (make-256-gray-level-image
		(image::read-image-file (merge-pathnames filename)))))
    (setf (canvas-image (rdoc-canvas *application-frame*)) image))
  (setf-status-line :filename filename))

(defun save-image (filename &key (format :binary))
  (when (canvas-image (rdoc-canvas *application-frame*))
    (image::write-pnm (canvas-image (rdoc-canvas *application-frame*)) 
		      filename format)
    (setf-status-line :filename filename)))

(define-command com-load-image ()
  (write-mini-buffer "Enter filename : ")
  (let ((filename (read-mini-buffer)))
    (if (probe-file filename)
	(load-image filename)
        (write-mini-buffer (format nil "~a doesn't exist !" filename)))))

(define-command com-save-image ()
  (write-mini-buffer "Enter filename : ")
  (let ((filename (read-mini-buffer :default-string 
				    (status-filename 
				     (rdoc-status *application-frame*)))))
    (if (probe-file filename)
	(progn
	  (write-mini-buffer "Do you want overwrite(o): ")
	  (let ((answer (read-mini-buffer)))
	    (if (string= answer "o")
		(save-image filename)
	        (write-mini-buffer (format nil "Save aborted.")))))
	(if (save-image filename)
	    (write-mini-buffer (format nil "File saved."))
	    (write-mini-buffer (format nil "Save aborted."))))))

(define-command com-exit ()
  (throw (rdoc-exit-exception *application-frame*) nil))

(defun com-histo ()
  (let* ((image (canvas-image (rdoc-canvas *application-frame*)))
	 (pixels (when image (clim-clx::image-pixels image))))
    (when pixels
      (make-gnuplot-histo pixels 
			  (format nil 
				  "~A-histo"
				  (status-filename 
				   (rdoc-status *application-frame*)))))))

(make-command-table 'menubar-command-table
                    :errorp nil
                    :menu '(("File" :menu file-command-table)
			    ("Modules" :menu modules-command-table)))

(make-command-table 'file-command-table
                    :errorp nil
                    :menu '(("Load image" :command com-load-image)
			    ("Save image" :command com-save-image)
                            ("Exit" :command com-exit)))

(make-command-table 'modules-command-table
                    :errorp nil)

(define-application-frame rdoc-frame ()
  ((exit-exception :initform (intern (concatenate 'string
						  (symbol-name (gensym))
						  "-exit"))
		   :reader rdoc-exit-exception)
   (tools :initform (make-hash-table :test #'equal) 
	  :accessor rdoc-tools)
   (current-tool :initform :hand 
		 :accessor rdoc-current-tool)
   (color-slider :initform nil 
		 :accessor rdoc-color-slider)
   (color-button :initform nil
		 :accessor rdoc-color-button)
   (canvas :initform nil 
	   :accessor rdoc-canvas)
   (interactor :initform nil 
	       :accessor rdoc-interactor)
   (status :initform nil 
	   :accessor rdoc-status)
   (mini-buffer :initform nil 
		:accessor rdoc-mini-buffer)
   (mini-buffer-active-p :initform nil 
			 :accessor rdoc-mini-buffer-active-p))
  (:panes
   (menu-bar (climi::make-menu-bar 'menubar-command-table :height 25))
   (canvas (make-pane 'canvas-pane 
		      :min-height 400 
		      :min-width 400 
		      :name 'canvas))
   (interactor :interactor 
	       :height 60 
	       :min-width 600 
	       :min-height 80)
   (status (make-pane 'status-line-pane 
		      :value "Ready..." 
		      :height 25 
		      :name 'status))
   (mini-buffer (make-clim-interactor-pane :height 25 
					   :scroll-bars nil 
					   :name 'mini-buffer))

   ; Tool box
   (hand-button (make-tool-button "Hand" :hand :value t))
   (region-button (make-tool-button "Region" :region))
   (pen-button (make-tool-button "Pen" :pen))
   (eraser-button (make-tool-button "Eraser" :eraser))
   (histo-button (make-tool-button "Histo" :histo 
				   :callback #'com-histo))
   (pipette-button (make-tool-button "Pipette" :pipette))
   (color-slider :slider 
		 :value 0
		 :min-value 0 :max-value 255
		 :value-changed-callback 'color-slider-callback
		 :drag-callback 'color-slider-callback
		 :show-value-p t
		 :decimal-places 0
		 :height 50 :width 50
		 :orientation :horizontal)
   (color-button (make-pane 'gray-pane
			    :gray-level 0
			    :min-height 50 :width 100 
			    :name 'color-button)))
  (:layouts
   (defaults (vertically (:min-height 500 :max-width 5000) 
	       menu-bar 
	       (horizontally (:min-width 600 :width 600)
		 (vertically (:width 100)		  
			     hand-button
			     region-button
			     pen-button
			     eraser-button
			     histo-button
			     pipette-button
			     color-slider
			     color-button)
		 (canvas-scrolling () canvas))
	       interactor 
	       status
	       mini-buffer)))
  (:top-level (rdoc-frame-top-level)))

(defmethod rdoc-frame-top-level ((frame application-frame) 
				 &key (prompt "* ") &rest rest)
  (declare (ignore rest))
  (when climi::*multiprocessing-p*
    (sleep 2))
  (catch (rdoc-exit-exception frame)
    (loop
     (let* ((interactor (rdoc-interactor frame))
	    (*standard-input* interactor)
	    results)
       (stream-write-string *standard-input* prompt)
       (stream-finish-output *standard-input*)
       (setq results ; eval interactor's content
	     (multiple-value-list (eval 
				   (read-from-string 
				    (my-stream-read-line *standard-input*)))))
       (loop for result in results 
	     do (print result *standard-input*)) 
       (terpri *standard-input*))))
  ;; make it until destroy-frame be implemented in McCLIM 
  (destroy-mirror (climi::port frame)  
		  (frame-top-level-sheet frame)))

