;;; -*- Mode: Lisp; Package: RDOC -*-

;;;  (c) copyright 2002 by
;;;           Joachim P., Jrme P., Hugh P., Matthias R.
;;;           (restordoc@emi.u-bordeaux.fr)
;;;
;;; This library is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU Library General Public
;;; License as published by the Free Software Foundation; either
;;; version 2 of the License, or (at your option) any later version.
;;;
;;; This library is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; Library General Public License for more details.
;;;
;;; You should have received a copy of the GNU Library General Public
;;; License along with this library; if not, write to the Free
;;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
;;; 02111-1307 USA.

(in-package :RDOC)

(defclass canvas-scroller-pane (climi::composite-pane)
  ((scroll-bar :type (member '(t :vertical :horizontal))
	       :initform t
	       :initarg :scroll-bar
	       :accessor scroller-pane-scroll-bar)
   (viewport   :initform nil)
   (vscrollbar :initform nil)
   (hscrollbar :initform nil)))

(defparameter *scrollbar-thickness* 15)

(defmacro canvas-scrolling ((&rest options) &body content)
  `(make-pane 'canvas-scroller-pane ,@options 
              :contents (list ,@content)))

;;; Layout

(defmethod compose-space ((pane canvas-scroller-pane))
  (with-slots (viewport vscrollbar hscrollbar) pane
    (if viewport
        (let ((req (compose-space viewport)))
          (when vscrollbar
            (setq req (space-requirement+* req
                                           :height     
					   *scrollbar-thickness*
                                           :min-height 
					   *scrollbar-thickness*
                                           :max-height 
					   *scrollbar-thickness*)))
          (when hscrollbar
            (setq req (space-requirement+* req
                                           :width 
					   *scrollbar-thickness*
                                           :min-width
					   *scrollbar-thickness*
                                           :max-width 
					   *scrollbar-thickness*)))
          req)
        (make-space-requirement))))

(defmethod allocate-space ((pane canvas-scroller-pane) width height)
  (with-slots (viewport vscrollbar hscrollbar) pane
    (when viewport
      (setf (sheet-transformation viewport)
	    (make-translation-transformation (if vscrollbar 
						 *scrollbar-thickness* 
						 0) 
					     0))
      (allocate-space viewport
		      (if vscrollbar 
			  (- width *scrollbar-thickness*) 
			  width)
		      (if hscrollbar 
			  (- height *scrollbar-thickness*) 
			  height)))
    (when vscrollbar
      (setf (sheet-transformation vscrollbar)
	    (make-translation-transformation 0 0))
      (allocate-space vscrollbar
		      *scrollbar-thickness*
		      (if hscrollbar 
			  (- height *scrollbar-thickness*) 
			  height)))
    (when hscrollbar
      (move-sheet hscrollbar
                  (if vscrollbar
                      *scrollbar-thickness*
                      0)
                  (- height *scrollbar-thickness*))
      (allocate-space hscrollbar
		      (if vscrollbar 
			  (- width *scrollbar-thickness*) 
			  width)
		      *scrollbar-thickness*))))

;;; Initialization

(defmethod initialize-instance :after ((pane canvas-scroller-pane) 
				       &key contents &allow-other-keys)
  (sheet-adopt-child pane (first contents))
  (with-slots (scroll-bar viewport vscrollbar hscrollbar) pane
    (setq viewport (first (sheet-children pane)))
    (unless (eq scroll-bar :horizontal)
      (setq vscrollbar (my-make-scroll-bar-pane pane 
						:vertical viewport))
      (sheet-adopt-child pane vscrollbar))
    (unless (eq scroll-bar :vertical)
      (setq hscrollbar (my-make-scroll-bar-pane pane 
						:horizontal viewport))
      (sheet-adopt-child pane hscrollbar))))

(defun my-make-scroll-bar-pane (pane orientation viewport)
  (make-pane 'scroll-bar-pane
	     :orientation orientation
	     :client (first (sheet-children viewport))
	     :drag-callback (lambda (gadget new-value)
			      (declare (ignore gadget))
			      (scroller-drag-callback pane 
						      orientation 
						      new-value))
	     :scroll-up-page-callback
	     #'(lambda (scroll-bar)
		 (scroll-page-callback scroll-bar 1))
	     :scroll-down-page-callback
	     #'(lambda (scroll-bar)
		 (scroll-page-callback scroll-bar -1))
	     :scroll-up-line-callback
	     #'(lambda (scroll-bar)
		 (scroll-line-callback scroll-bar 1))
	     :scroll-down-line-callback
	     #'(lambda (scroll-bar)
		 (scroll-line-callback scroll-bar -1))
	     :min-value 0
	     :max-value 1))

;;; Scrolling itself :  Callbacks

(defmethod scroller-drag-callback ((pane canvas-scroller-pane) 
				   orientation new-value)
  (with-slots (viewport hscrollbar vscrollbar) pane
    (let ((image (canvas-image (rdoc-canvas *application-frame*)))
	  (scrollbar)
	  (dimension))
      (if (eq orientation :vertical)
	  (setf scrollbar vscrollbar
		dimension (image-height image))
	  (setf scrollbar hscrollbar
		dimension (image-width image)))
      (with-bounding-rectangle* (x1 y1 x2 y2) (sheet-region viewport)
	(let ((dif (- (gadget-value scrollbar) new-value)))
	  (scroll-line-callback scrollbar (* dimension dif)))))))

(defun scroll-page-callback (scroll-bar direction)
  (scroll-line-callback scroll-bar (* 10 direction)))

(defun scroll-line-callback (scroll-bar direction)
  (with-slots (climi::client climi::orientation climi::value) scroll-bar
    (setf climi::value (+ (* (- direction) .10) 
			  climi::value))
    (let ((canvas-pane (rdoc-canvas *application-frame*)))
      (with-slots (translate-x translate-y) canvas-pane
	(if (eq climi::orientation :vertical)
	    (incf translate-y (* direction 10))
	    (incf translate-x (* direction 10)))
	(set-transformation canvas-pane)
	(dispatch-repaint canvas-pane nil)))))
